GateLayer

Struct GateLayer 

pub struct GateLayer<F: Field> {
    pub layer_id: LayerId,
    pub num_dataparallel_vars: usize,
    pub nonzero_gates: Vec<(u32, u32, u32)>,
    pub lhs: DenseMle<F>,
    pub rhs: DenseMle<F>,
    pub phase_1_mles: Option<Vec<Vec<DenseMle<F>>>>,
    pub phase_2_mles: Option<Vec<Vec<DenseMle<F>>>>,
    pub gate_operation: BinaryOperation,
    beta_g2_vec: Option<Vec<BetaValues<F>>>,
    g_vec: Option<Vec<Vec<F>>>,
    num_rounds_phase1: usize,
}
Generic gate struct – the binary operation performed by the gate is specified by the gate_operation parameter. Additionally, the number of dataparallel variables is specified by num_dataparallel_vars in order to account for batched and un-batched gates.

Fields

layer_id: LayerId

The layer id associated with this gate layer.

num_dataparallel_vars: usize

The number of bits representing the number of “dataparallel” copies of the circuit.

nonzero_gates: Vec<(u32, u32, u32)>

A vector of tuples representing the “nonzero” gates, especially useful in the sparse case the format is (z, x, y) where the gate at label z is the output of performing an operation on gates with labels x and y.

lhs: DenseMle<F>

The left side of the expression, i.e. the mle that makes up the “x” variables.

rhs: DenseMle<F>

The right side of the expression, i.e. the mle that makes up the “y” variables.

phase_1_mles: Option<Vec<Vec<DenseMle<F>>>>

The mles that are constructed when initializing phase 1 (binding the x variables).

phase_2_mles: Option<Vec<Vec<DenseMle<F>>>>

The mles that are constructed when initializing phase 2 (binding the y variables).

gate_operation: BinaryOperation

The gate operation representing the fan-in-two relationship.

beta_g2_vec: Option<Vec<BetaValues<F>>>

the beta table which enumerates the incoming claim’s challenge points on the dataparallel vars of the MLE

g_vec: Option<Vec<Vec<F>>>

The incoming claim’s challenge points.

num_rounds_phase1: usize

the number of rounds in phase 1

Implementations

Source

impl<F: Field> GateLayer<F>

pub fn new( num_dataparallel_vars: Option<usize>, nonzero_gates: Vec<(u32, u32, u32)>, lhs: DenseMle<F>, rhs: DenseMle<F>, gate_operation: BinaryOperation, layer_id: LayerId, ) -> Self

Construct a new gate layer

Arguments
  • num_dataparallel_vars: an optional representing the number of bits representing the circuit copy we are looking at.

None if this is not dataparallel, otherwise specify the number of bits

  • nonzero_gates: the gate wiring between single-copy circuit (as the wiring for each circuit remains the same)

x is the label on the batched mle lhs, y is the label on the batched mle rhs, and z is the label on the next layer, batched

  • lhs: the flattened mle representing the left side of the summation
  • rhs: the flattened mle representing the right side of the summation
  • gate_operation: which operation the gate is performing. right now, can either be an ‘add’ or ‘mul’ gate
  • layer_id: the id representing which current layer this is
Returns

A Gate struct that can now prove and verify rounds

fn init_phase_1(&mut self, challenges: Vec<F>)

Initialize phase 1, or the necessary mles in order to bind the variables in the lhs of the expression. Once this phase is initialized, the sumcheck rounds binding the “x” variables can be performed.

fn init_phase_1_rlc(&mut self, challenges: &[&[F]], random_coefficients: &[F])

fn init_phase_2(&mut self, u_claim: &[F], f_at_u: F, g1_claim_points: &[F])

Initialize phase 2, or the necessary mles in order to bind the variables in the rhs of the expression. Once this phase is initialized, the sumcheck rounds binding the “y” variables can be performed.

fn init_phase_2_rlc( &mut self, u_claim: &[F], f_at_u: F, g1_claim_points: &[&[F]], random_coefficients: &[F], )