pub struct VerifierGateLayer<F: Field> {
layer_id: LayerId,
gate_operation: BinaryOperation,
wiring: Vec<(u32, u32, u32)>,
lhs_mle: VerifierMle<F>,
rhs_mle: VerifierMle<F>,
claim_challenge_points: Vec<Vec<F>>,
num_dataparallel_rounds: usize,
dataparallel_sumcheck_challenges: Vec<F>,
first_u_challenges: Vec<F>,
last_v_challenges: Vec<F>,
}Expand description
The verifier’s counterpart of a Gate layer.
Fields
layer_id: LayerIdThe layer id associated with this gate layer.
gate_operation: BinaryOperationThe gate operation representing the fan-in-two relationship.
wiring: Vec<(u32, u32, u32)>A vector of tuples representing the “nonzero” gates, especially useful in the sparse case the format is (z, x, y) where the gate at label z is the output of performing an operation on gates with labels x and y.
lhs_mle: VerifierMle<F>The left side of the expression, i.e. the mle that makes up the “x” variables.
rhs_mle: VerifierMle<F>The mles that are constructed when initializing phase 2 (binding the y variables).
claim_challenge_points: Vec<Vec<F>>The challenge points for the claim on the VerifierGateLayer.
num_dataparallel_rounds: usizeThe number of dataparallel rounds.
dataparallel_sumcheck_challenges: Vec<F>The challenges for p_2, as derived from sumcheck.
first_u_challenges: Vec<F>The challenges for x, as derived from sumcheck.
last_v_challenges: Vec<F>The challenges for y, as derived from sumcheck.
Implementations
Sourceimpl<F: Field> VerifierGateLayer<F>
impl<F: Field> VerifierGateLayer<F>
Sourcepub fn evaluate(&self, claims: &[&[F]], random_coefficients: &[F]) -> F
pub fn evaluate(&self, claims: &[&[F]], random_coefficients: &[F]) -> F
Computes the oracle query’s value for a given VerifierGateLayer.